import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;


public class MainReport {
	final static String driverName = "com.mysql.jdbc.Driver";
	final static String host = "localhost";
	final static String user = "root";
	final static String pass = "";
	final static String database = "jiradb";
	
	public static Hashtable contrats = new Hashtable();

	/*
	 * Tableaux  deux dimensions qui dfinissent les indicateurs contractuels
	 * 1ere dim : correspond  la priorit de 0  3
	 * 2eme dim : dfinit la dure au format 0 => jours, 1 => heures, 2 => minutes
	 */
	final static int TMAC01[][] = {{0,0,5,0}, {0,1,0,0}, {0,8,0,1}, {5,0,0,1} };
	final static int TMAC02[][] = {{0,2,0,0}, {0,4,0,0}, {0,8,0,1}, {0,0,0,0}};
	final static int TMAC03[][] = {{0,24,0,0}, {5,0,0,1}, {10,0,0,1}, {20,0,0,1}};

	private ArrayList myData;
	private ArrayList projects;
	
	public MainReport() {
		this.myData = new ArrayList();
		this.projects = new ArrayList();
		
		contrats.put("houdini", "'ALLOC' ,'RDW' , 'REIME', 'REIMM', 'RESA', 'RIM', 'RMS', 'ORPM', 'OSIM'");
	}
	
	public ArrayList getMyData() {
		return myData;
	}

	public void setMyData(ArrayList myData) {
		this.myData = myData;
	}

	public ArrayList getProjects() {
		return projects;
	}

	public void setProjects(ArrayList projects) {
		this.projects = projects;
	}

	public void loadData(String contract, String debut, String fin) {
		try {
			Connection conn = this.getConnection();
			
			Statement stmt = null;
			ResultSet rs = null;
			ResultSet rsBis = null;
			ResultSetMetaData rsmd = null;
			
			/* Variables Temporaires */
			String pnameTmp;
			String pkeyTmp;
			int stepIdTmp;
			String priorityTmp;
			Date creationTmp;
			Date updatedTmp;
			Date startDateTmp;
			Date finishDateTmp;
			
			String currentProject = null;
			String currentIssue = null;
			boolean isFirst = true;
			
			Project aProject = null;
			Issue anIssue = null;
				
			try {
				stmt = conn.createStatement();
							
				String query = 	"SELECT project.pname, jiraissue.pkey, jiraissue.created, jiraissue.updated, jiraissue.PRIORITY, os_historystep.step_id, os_historystep.start_date, os_historystep.finish_date "+
								"FROM project, jiraissue, os_historystep "+
								"WHERE project.pkey "+
								"IN ("+contrats.get(contract)+") "+
								"AND jiraissue.created "+
								"BETWEEN \""+debut+"\" AND \""+fin+"\" "+ 
								"AND jiraissue.workflow_id = os_historystep.entry_id "+
								"AND jiraissue.project = project.id "+
								"AND os_historystep.step_id "+
								"IN ( 1, 2, 3, 12 ) "+
								"ORDER BY jiraissue.pkey";
								
				rs = stmt.executeQuery(query);
				rsmd = rs.getMetaData();
				
				while(rs.next()) {	
					pnameTmp = rs.getString("pname");
					pkeyTmp = rs.getString("pkey");
					
					/* Initialisation des variables tmoins lors du premier passage */
					if(isFirst) {
						System.out.println("Initialisation");
						currentProject = pnameTmp;
						currentIssue = pkeyTmp;
						
						/* Cration d'un projet */
						aProject = new Project();
						aProject.setPname(pnameTmp);
						
						/* Cration d'une demande */
						anIssue = new Issue();
						anIssue.setPkey(pkeyTmp);
						anIssue.setCreation(rs.getTimestamp("created"));
						anIssue.setUpdated(rs.getTimestamp("updated"));
						anIssue.setPriorite(this.verifPriorite(rs.getString("priority")));
						
						isFirst = false;
					}
							
					/* Changement de demande mais pas de projets */
					if(!pkeyTmp.equals(currentIssue) && pnameTmp.equals(currentProject)) {
						
						System.out.println("Changement de demande");
						anIssue.calculTMACs();
						aProject.getIssues().add(anIssue);
						
						anIssue = new Issue();
						anIssue.setPkey(pkeyTmp);
						anIssue.setCreation(rs.getTimestamp("created"));
						anIssue.setUpdated(rs.getTimestamp("updated"));
						anIssue.setPriorite(this.verifPriorite(rs.getString("priority")));
						
						currentIssue = pkeyTmp;
					}
					
					/* Changement de projet */
					else if(!pnameTmp.equals(currentProject)) {
						System.out.println("Changement de projet");
						anIssue.calculTMACs();
						aProject.getIssues().add(anIssue);
						this.getProjects().add(aProject);
						
						aProject = new Project();
						aProject.setPname(pnameTmp);
						
						anIssue = new Issue();
						anIssue.setPkey(pkeyTmp);
						anIssue.setPriorite(this.verifPriorite(rs.getString("priority")));
						anIssue.setCreation(rs.getTimestamp("created"));
						anIssue.setUpdated(rs.getTimestamp("updated"));
						
						currentIssue = pkeyTmp;
						currentProject = pnameTmp;
					}
					
					StepIssue aStep = new StepIssue();
					aStep.setStepId(rs.getInt("step_id"));
					aStep.setDebut(rs.getTimestamp("start_date")); //("start_date");
					aStep.setFin(rs.getTimestamp("finish_date"));
					
					aStep.init();
					anIssue.addTMADelay(aStep);
					anIssue.getEtapes().add(aStep);
					
					System.out.println("Etape : "+aStep.getStepId()+"\tDuree : "+aStep.displayPeriod(aStep.getPeriode()));
					
					System.out.println("Projet : "+pnameTmp+"\tDemande : "+pkeyTmp+"\tNbIssue :"+anIssue.getEtapes().size());
				}
				
				/* Ajoute des dernieres demandes et du dernier projets */
				aProject.getIssues().add(anIssue);
				this.getProjects().add(aProject);
    		} finally {
	    		rs.close();
	    		stmt.close();
	    	}
		    
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	/**
	 * Permet de crer la connexion  la BD
	 * @return Connection
	 * @throws Exception
	 */
	public Connection getConnection() throws Exception {
		
		Connection conn = null;
		
		try {
	        System.out.println("Connection au driver JDBC");
	        Class.forName(driverName).newInstance();
	  
	        try {
	         
	          conn = 
	        	  DriverManager.getConnection("jdbc:mysql://"+host+"/"+database+"?user="+user+"&password="+pass);
	          System.out.println("Base de donnes connecte");
	          
	          
	        } catch (SQLException ex) {
	          // la connection a la base de donnes n'a pas pu etre tabli
	          // voici les codes erreurs retourns 
	          System.out.println("SQLException: " + ex.getMessage()); 
	          System.out.println("SQLState: " + ex.getSQLState()); 
	          System.out.println("VendorError: " + ex.getErrorCode()); 
	        }
	    } catch (Exception ex) { 
	        // Le driver n'a pas pu tre charg
	        // vrifier que la variable CLASSPATH est bien renseigne
	        System.out.println("Echec de chargement du driver");
	    }
	    
	    return conn;
	}
	
	/**
	 * Permet de faire correspondre les priorits de JIRA avec celles du contrat
	 * @param jiraPriorite
	 * @return String : la bonne priorit correspondante
	 */
	public String verifPriorite(String jiraPriorite) {
		String contratPriorite = new String();
		if(jiraPriorite.equals("1") || jiraPriorite.equals("2") || jiraPriorite.equals("3"))
			contratPriorite = jiraPriorite;
		else if(jiraPriorite.equals("4") || jiraPriorite.equals("5"))
			contratPriorite = "4";
			
		return contratPriorite;
	}
	
	public void readData() {
	
		for(int i = 0; i < this.getProjects().size(); i++) {
			Project p = (Project) this.getProjects().get(i);
	
			for(int j = 0; j < p.getIssues().size(); j++) {
				Issue anIssue = (Issue) p.getIssues().get(j);
				
				String [] myStrData = {p.getPname(), anIssue.getPkey(), 
										anIssue.getPriorite(),DateFunction.sdf.format(anIssue.getCreation()), 
										DateFunction.sdf.format(anIssue.getUpdated()), StepIssue.displayPeriod(anIssue.getTMAC01()),
										StepIssue.displayPeriod(anIssue.getTMAC02()), StepIssue.displayPeriod(anIssue.getTMAC03())};
				
				this.getMyData().add(myStrData);
				
				/*System.out.println(	"Projet : "+p.getPname()+"\tDemande : "+anIssue.getPkey()+
									"\tPriorite : "+anIssue.getPriorite()+"\tNbEtapes : "+anIssue.getEtapes().size()+
									"\tTMAC01 : "+StepIssue.displayPeriod(anIssue.getTMAC01())+"\tTMAC03 : "+StepIssue.displayPeriod(anIssue.getTMAC03()));
				
				for(int k =0; k<anIssue.getEtapes().size(); k++) {
					StepIssue aStep = (StepIssue) anIssue.getEtapes().get(k);
					System.out.println(	"Projet : "+p.getPname()+"\tDemande : "+anIssue.getPkey()+"\tPriorite : "+anIssue.getPriorite()+
										"\tEtape id : "+aStep.getStepId()+"\tDure : "+aStep.displayPeriod(aStep.getPeriode()));
				}*/
			}
		}
	}
}
